function [sys,x0,str,ts] = IL_S_fun_pre(t,x,u,flag)
% x is the integrate tracking error
% u is command xc, and plant state x
% u vector contains commands (not prefiltered) (5), and plant states (12)
persistent y k A B g

switch flag
  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%

  case 0        
      [sys,x0,str,ts] = mdlInitializeSizes;
      y = zeros(3,1);
      k = 0;
      A = zeros(2,1);
      B = eye(2,2);
      g = [0;1];
      
  %%%%%%%%%%%%%%%
  % Derivatives %
  %%%%%%%%%%%%%%%

  case 1
    sys = mdlDerivatives(t,x,u,y,k,A,B,g);

  %%%%%%%%%%%%%%%%%%%%%%%%
  % Update  %
  %%%%%%%%%%%%%%%%%%%%%%%%
  
  case {2}
    sys = []; % do nothing

  %%%%%%%%%%%%%%%%%%%%%%%%
  % Terminate %
  %%%%%%%%%%%%%%%%%%%%%%%%
  case {9}
    sys = mdlTerminate(t,x,u);

  %%%%%%%%%%
  % Output %
  %%%%%%%%%%

  case 3
    [sys,y,k,A,B,g] = mdlOutputs(t,x,u); 

otherwise
    error(['unhandled flag = ',num2str(flag)]);
end



% end limintm



%=============================================================================

% mdlInitializeSizes

% Return the sizes, initial conditions, and sample times for the S-function.

%=============================================================================

%

function [sys,x0,str,ts] = mdlInitializeSizes
%
% call simsizes for a sizes structure, fill it in and convert it to a
% sizes array
%
global Radius

sizes = simsizes;
                               %1   2    3       4   5  6  7   8     9 
sizes.NumContStates  = 15;   % xi_x,xi_y,xi_psi,xc,dxc,yc,dyc,psi_c,dpsi_c,uc,duc,rc,drc
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 2+3+3+sizes.NumContStates+2+1+1;   % F,tau,psi_co,u_co,r_co,psi_bs,u_bs,r_bs,x,Lyap_tilde,Lyap_nu
sizes.NumInputs      = 2+5;   % x_co,y_co,u,r,x,y,psi
sizes.DirFeedthrough = 1;   % If MATLAB 6.0 and up need to use 1, otherwise 0
sizes.NumSampleTimes = 1;     % at least one sample time is needed
sys = simsizes(sizes);

% initialize the initial conditions
%
x0 = zeros(sizes.NumContStates,1);

x0(4)  = Radius - .2;

%
% str is always an empty matrix
%
str = [];

%
% initialize the array of sample times
%
ts  = [0,0];    

% end mdlInitializeSizes



%

%==========================================================================
%===
% mdlDerivatives
% Compute derivatives for continuous states.
%==========================================================================
%===
%
function [sys] = mdlDerivatives(t,x,u,y,k,A,B,g)
%                 1  ,  2 ,  3   , 4, 5 , 6,  7,    8,    9, 10, 11,12, 13
% state order: % zeta_x,zeta_y,zeta_psi,xc,dxc,yc,dyc,psi_c,dpsi_c,uc,duc,rc,drc

global K_xy x_wn x_zeta y_wn y_zeta u_wn u_zeta r_wn r_zeta
global psi_wn psi_zeta K_psi
global u_wn u_zeta r_wn r_zeta

x_co    =  u(1);
y_co    =  u(2);
psi_co  =  y(1);
u_co    =  y(2);
r_co    =  y(3);

u_xc = x(10)*cos(x(8)); %u_c cos(psi_c)
u_yc = x(10)*sin(x(8)); %u_c sin(psi_c)
ux_co = u_co*cos(psi_co); %u_co cos(psi_co)
uy_co = u_co*sin(psi_co); %u_co sin(psi_co)

% compute derivatives of zeta
Bg = B*g;
ABg = [A,Bg];
ABg_err = ABg*[0;x(3)];
sys(1) = -k*x(1) + ABg_err(1) + (u_xc - ux_co);
sys(2) = -k*x(2) + ABg_err(2) + (u_yc - uy_co);
sys(3) = -K_psi*x(3) + (x(12) - r_co) - 0; % eqn 25.


% filter x_co and y_co to get x_c, x_c_dot and y_c, y_c_dot
sys(4) = x(5);                     % x(2) = x_c
sys(5) = x_wn^2*(x_co-x(4)) - 2*x_wn*x_zeta*x(5);  % x(1) = x_c_dot 
sys(6) = x(7);                     % x(4) = y_c
sys(7) = y_wn^2*(y_co-x(6)) - 2*y_wn*y_zeta*x(7);  % x(3) = y_c_dot 

% filter psi_co to get psi_c and psi_c_dot 
sys(8) = x(9);                     % x(2) = psi_c
sys(9) = psi_wn^2*twopilimit(psi_co- x(8)) - 2*psi_wn*psi_zeta*x(9) ;  % u(6) = psi_co, x(1) = psi_c_dot

% filter u_co to get u_c and u_c_dot 
sys(10) = x(11);      
sys(11) = u_wn^2*(u_co-x(10)) - 2*u_wn*u_zeta*x(11); 

% filter r_co to get r_c and r_c_dot 
sys(12) = x(13);                    
sys(13) = r_wn^2*(r_co-x(12)) - 2*r_wn*r_zeta*x(13); 

% compute and integrate the tracking error
u_tilde = u(3) - x(10);
sys(14) = u_tilde;
r_tilde = u(4) - x(12);
sys(15) = r_tilde;
% end mdlDerivatives



%

%==========================================================================
%===
% mdlOutputs
% Return the output vector for the S-functkon
%==========================================================================
%===
%

function [sys,y,k,A,B,g] = mdlOutputs(t,x_cntrl,x_plnt)

global K_u K_r Ki_u Ki_r K_psi K_xy Ku_hat Kuu_hat Kr_hat Krr_hat
gama = 1; % select direction (forward or backward) by sign of gama
% if t>20,  % try for fun
%     gama = -1;
% end
% read out plant state to make code easy to read
u   = x_plnt(3);
r   = x_plnt(4);
x   = x_plnt(5);
y   = x_plnt(6);
psi   = x_plnt(7);

%read out control state to make code easy to read
zeta_x  = x_cntrl(1);
zeta_y  = x_cntrl(2);
zeta_psi  = x_cntrl(3);
zeta_u  = 0;            % for completeness
zeta_r  = 0;            % for completeness
x_c     = x_cntrl(4);
dxc     = x_cntrl(5);
y_c     = x_cntrl(6);
dyc     = x_cntrl(7);
psi_c   = x_cntrl(8);
dpsic   = x_cntrl(9);
u_c     = x_cntrl(10);
duc     = x_cntrl(11);
r_c     = x_cntrl(12);
drc     = x_cntrl(13);
iu_tilde= x_cntrl(14);
ir_tilde= x_cntrl(15);

% compute tracking errors
x_tilde     = x - x_c; 
y_tilde     = y - y_c; 
psi_tilde   = twopilimit(psi - psi_c);
u_tilde     = u - u_c; 
r_tilde     = r - r_c; 
tilde      = [x_tilde,y_tilde,psi_tilde,u_tilde,r_tilde]';
Lyap_tilde = 0.5*tilde'*tilde;

% compute compenstated tracking errors
x_nu    = x_tilde - zeta_x; 
y_nu    = y_tilde - zeta_y; 
psi_nu  = psi_tilde - zeta_psi;
u_nu    = u_tilde - zeta_u; 
r_nu    = r_tilde - zeta_r; 
%nu      = [x_nu,y_nu,psi_nu,u_nu,r_nu]';
%Lyap_nu = 0.5*nu'*nu;
% with the integrator
p1 = 1; p2 = 1;
nu      = [x_nu,y_nu,psi_nu,p1*u_nu,p2*r_nu,p1*iu_tilde,p2*ir_tilde]';
Lyap_nu = 0.5*nu'*nu;

%compute bs terms
A = [cos(psi_c);sin(psi_c)];
B = u(1)*[cos(psi_c) -sin(psi_c);
          sin(psi_c)  cos(psi_c)];
nu_xy = [x_nu;y_nu];
if psi_tilde ~= 0,  % (from Eqn. 38)
    g = [(cos(psi_tilde)-1)/psi_tilde; sin(psi_tilde)/psi_tilde];
else
    g = [ 0;  1];
end
psi_bs = g'*B'*nu_xy; % Eqn. 32
u_bs   = A'*nu_xy; % Eqn. 33
r_bs   = psi_nu;

%Control Law
k     = compute_f_4_xy(K_xy,[dxc;dyc],[x_tilde;y_tilde]);  % select position control gain
ux_co = -k*x_tilde + dxc;
uy_co = -k*y_tilde + dyc;
u_co  = gama*sqrt(ux_co*ux_co + uy_co*uy_co); % Eqn. 14
psi_co = atan2(gama*uy_co, gama*ux_co);       % Eqn. 15
r_co = -K_psi*psi_tilde + dpsic - psi_bs; 
g_hat = Ku_hat*u + Kuu_hat*abs(u)*u;
f_hat = Kr_hat*r + Krr_hat*abs(r)*r;
F    = -g_hat -K_u*u_tilde -1*Ki_u*iu_tilde + duc - u_bs;
Tau  = -f_hat -K_r*r_tilde -1*Ki_r*ir_tilde + drc - r_bs;

y = [psi_co;u_co;r_co];
sys=[F;Tau;y;psi_bs;u_bs;r_bs;x_cntrl;Lyap_tilde;Lyap_nu;k;psi_nu]';

% end mdlOutputs%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%







%==========================================================================
%===
% mdlTerminate
% Plot the graph of adaptive bound
%==========================================================================
%===
%
function sys = mdlTerminate(t,x,u,center)
    sys = [];

% end mdlTerminate





% Selection of the proportional gain (Appendix III)
function [k]=compute_f_4_xy(K_xy,vd,E)
% vd is the tangent plane desired velocity vector
% E is the tangent plane position error vector
alpha = 0.5;       
dot_product = vd'*E;
norm_sq     = vd'*vd;
nrm_E       = sqrt(E'*E);

max_delta_spd = 1;  % maximum allowable change in speed that the controller can use to fix pos error
if K_xy*nrm_E > max_delta_spd,  % Adjust gain to not exceed max pert speed
    K_xy = max_delta_spd/nrm_E;
end

if (dot_product > 0)
    k  = alpha * (norm_sq/dot_product);
    k  = min(k, K_xy);
else
    k = K_xy;
end





